﻿using AutoWorkshopBusinessLogic.OfficePackage;
using AutoWorkshopBusinessLogic.OfficePackage.HelperModels;
using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopContracts.ViewModels;

namespace AutoWorkshopBusinessLogic.BusinessLogics
{
    public class ReportLogic : IReportLogic
    {
        private readonly IComponentStorage _componentStorage;
        private readonly IRepairStorage _RepairStorage;
        private readonly IOrderStorage _orderStorage;
        private readonly AbstractSaveToExcel _saveToExcel;
        private readonly AbstractSaveToWord _saveToWord;
        private readonly AbstractSaveToPdf _saveToPdf;

        public ReportLogic(IRepairStorage RepairStorage, IComponentStorage ComponentStorage, IOrderStorage OrderStorage,
            AbstractSaveToExcel SaveToExcel, AbstractSaveToWord SaveToWord, AbstractSaveToPdf SaveToPdf)
        {
            _RepairStorage = RepairStorage;
            _componentStorage = ComponentStorage;
            _orderStorage = OrderStorage;

            _saveToExcel = SaveToExcel;
            _saveToWord = SaveToWord;
            _saveToPdf = SaveToPdf;
        }

        public List<ReportRepairComponentViewModel> GetRepairComponents()
        {
            return _RepairStorage.GetFullList().
                Select(x => new ReportRepairComponentViewModel
                {
                    RepairName = x.RepairName,
                    Components = x.RepairComponents.Select(x => (x.Value.Item1.ComponentName, x.Value.Item2)).ToList(),
                    TotalCount = x.RepairComponents.Select(x => x.Value.Item2).Sum()
                })
                .ToList();
        }

        public List<ReportOrdersViewModel> GetOrders(ReportBindingModel Model)
        {
            return _orderStorage.GetFilteredList(new OrderSearchModel { DateFrom = Model.DateFrom, DateTo = Model.DateTo })
                .Select(x => new ReportOrdersViewModel
                {
                    Id = x.Id,
                    DateCreate = x.DateCreate,
                    RepairName = x.RepairName,
                    Sum = x.Sum,
                    Status = x.Status.ToString()
                })
                .ToList();
        }

        public void SaveRepairsToWordFile(ReportBindingModel Model)
        {
            _saveToWord.CreateDoc(new WordInfo
            {
                FileName = Model.FileName,
                Title = "Список ремонтов",
                Repairs = _RepairStorage.GetFullList()
            });
        }

        public void SaveRepairComponentToExcelFile(ReportBindingModel Model)
        {
            _saveToExcel.CreateReport(new ExcelInfo
            {
                FileName = Model.FileName,
                Title = "Список ремонтов",
                RepairComponents = GetRepairComponents()
            });
        }

        public void SaveOrdersToPdfFile(ReportBindingModel Model)
        {
            _saveToPdf.CreateDoc(new PdfInfo
            {
                FileName = Model.FileName,
                Title = "Список заказов",
                DateFrom = Model.DateFrom!.Value,
                DateTo = Model.DateTo!.Value,
                Orders = GetOrders(Model)
            });
        }
    }
}
