﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopFileImplement.Models;
using System;

namespace AutoWorkshopFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton _source;
        
        public OrderStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }

        public List<OrderViewModel> GetFullList()
        {
            return _source.Orders.Select(x => AddRepairName(x.GetViewModel)).ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel Model)
        {
            if (Model.DateFrom.HasValue)
            {
                return _source.Orders
                    .Where(x => (x.DateCreate >= Model.DateFrom && x.DateCreate <= Model.DateTo) && (!Model.ClientId.HasValue || x.ClientId == Model.ClientId)
                        && (!Model.ImplementerId.HasValue || x.ImplementerId == Model.ImplementerId))
                    .Select(x => AddImplementerFIO(AddClientFIO(AddRepairName(x.GetViewModel))))
                    .ToList();
            }

            return _source.Orders
                .Where(x => (Model.Id.HasValue && x.Id == Model.Id) || (Model.ClientId.HasValue && x.ClientId == Model.ClientId)
                    || (Model.ImplementerId.HasValue && x.ImplementerId == Model.ImplementerId))
                .Select(x => AddImplementerFIO(AddClientFIO(AddRepairName(x.GetViewModel))))
                .ToList();
        }

        public OrderViewModel? Delete(OrderBindingModel Model)
        {
            var Element = _source.Orders.FirstOrDefault(x => x.Id == Model.Id);
            
            if (Element != null)
            {
                _source.Orders.Remove(Element);
                _source.SaveOrders();
                
                return AddRepairName(Element.GetViewModel);
            }
            
            return null;
        }
        
        public OrderViewModel? GetElement(OrderSearchModel Model)
        {
            if (!Model.Id.HasValue && !Model.ImplementerId.HasValue && !Model.Status.HasValue)
                return null;

            var Order = _source.Orders.FirstOrDefault(x => 
                (Model.Id.HasValue && x.Id == Model.Id)
                || (Model.ImplementerId.HasValue && Model.Status.HasValue && x.ImplementerId == Model.ImplementerId && x.Status == Model.Status) );
            
            if (Order == null)
                return null;

            return AddImplementerFIO(AddClientFIO(AddRepairName(Order.GetViewModel)));
        }
        
        public OrderViewModel? Insert(OrderBindingModel Model)
        {
            Model.Id = _source.Orders.Count > 0 ? _source.Orders.Max(x => x.Id) + 1 : 1;
            
            var NewOrder = Order.Create(Model);            
            
            if (NewOrder == null)
                return null;

            _source.Orders.Add(NewOrder);
            _source.SaveOrders();
            
            return AddRepairName(NewOrder.GetViewModel);
        }
        
        public OrderViewModel? Update(OrderBindingModel Model)
        {
            var Order = _source.Orders.FirstOrDefault(x => x.Id == Model.Id);
            
            if (Order == null)
                return null;
 
            Order.Update(Model);
            _source.SaveOrders();
            
            return AddRepairName(Order.GetViewModel);
        }

        private OrderViewModel AddRepairName(OrderViewModel Model)
        {
            var SelectedRepair = _source.Repairs.FirstOrDefault(x => x.Id == Model.RepairId);

            Model.RepairName = SelectedRepair?.RepairName ?? string.Empty;
            return Model;
        }

        public OrderViewModel AddClientFIO(OrderViewModel Model)
        {
            var SelectedClient = _source.Clients.FirstOrDefault(x => x.Id == Model.Id);

            Model.ClientFIO = SelectedClient?.ClientFIO ?? string.Empty;
            return Model;
        }

        public OrderViewModel AddImplementerFIO(OrderViewModel Model)
        {
            var SelectedImplementer = _source.Implementers.FirstOrDefault(x => x.Id == Model.ImplementerId);

            if (SelectedImplementer != null)
            {
                Model.ImplementerFIO = SelectedImplementer.ImplementerFIO;
            }
            return Model;
        }
    }
}
