﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopDataModels.Models;
using System.Xml.Linq;

namespace AutoWorkshopFileImplement.Models
{
	public class MessageInfo : IMessageInfoModel
	{
		public string MessageId { get; set; } = string.Empty;

		public int? ClientId { get; set; }

		public string SenderName { get; set; } = string.Empty;

		public DateTime DateDelivery { get; set; }

		public string Subject { get; set; } = string.Empty;

		public string Body { get; set; } = string.Empty;

		public static MessageInfo? Create(MessageInfoBindingModel? Model)
		{
			if (Model == null)
				return null;

			return new()
			{
				MessageId = Model.MessageId,
				ClientId = Model.ClientId,
				SenderName = Model.SenderName,
				DateDelivery = Model.DateDelivery,
				Subject = Model.Subject,
				Body = Model.Body
			};
		}

		public MessageInfoViewModel GetViewModel => new()
		{
			MessageId = MessageId,
			ClientId = ClientId,
			SenderName = SenderName,
			DateDelivery = DateDelivery,
			Subject = Subject,
			Body = Body
		};

		public static MessageInfo? Create(XElement Element)
		{
			if (Element == null)
				return null;

			return new()
			{
				MessageId		= Element.Attribute("MessageId")!.Value,
				ClientId		= Convert.ToInt32(Element.Attribute("ClientId")!.Value),
				SenderName		= Element.Attribute("SenderName")!.Value,
				DateDelivery	= Convert.ToDateTime(Element.Attribute("DateDelivery")!.Value),
				Subject			= Element.Attribute("Subject")!.Value,
				Body			= Element.Attribute("Body")!.Value,
			};
		}

		public XElement GetXElement => new(
			"MessageInfo",
			new XAttribute("MessageId", MessageId),
			new XAttribute("ClientId", ClientId),
			new XAttribute("SenderName", SenderName),
			new XAttribute("DateDelivery", DateDelivery),
			new XAttribute("Subject", Subject),
			new XAttribute("Body", Body)
		);
	}
}
